from pydantic import BaseModel,EmailStr
from datetime import datetime

class Message(BaseModel):
    id: int
    sender: str
    recipient: str
    content: str
class MailerDto(BaseModel):
    recipients: list[EmailStr]
    subject: str
    message: str
    create_at: datetime
    is_html: bool


from pydantic import BaseModel
from typing import List, Optional
from datetime import datetime
from espy_contact.util.enums import ResourceEnum, GradeLevel
from espy_contact.model.campus import Term
import uuid


class Resource(BaseModel):
    """Type of resource can be Poll, Form Builder, Questionnaire, RichText, Video, Audio, File, Hyperlink."""

    id: Optional[str] = str(uuid.uuid4())
    title: str
    type: ResourceEnum
    lesson_id: str


class Lesson_note(BaseModel):
    id: Optional[str] = str(uuid.uuid4())
    title: str
    content: str
    lesson_id: Optional[str] = None


class Quiz(BaseModel):
    id: Optional[str] = str(uuid.uuid4())
    title: str
    question: str
    options: List[str]
    answer: str
    lesson_id: Optional[str] = None


class Lesson(BaseModel):  # defintion of biology, branches of biology
    id: Optional[str] = str(uuid.uuid4())
    title: str  # Intro to Biology
    quiz: Optional[Quiz] = None
    note: Optional[Lesson_note] = None
    assets: Optional[
        List[str]
    ] = []  # these are assets shared between users not same as resources
    resources: Optional[List[ResourceEnum]] = []
    topic_id: Optional[str] = None


class LessonDto(Lesson):
    class_id: Optional[str] = None


class Topic(BaseModel):  # Introduction to Biology
    id: Optional[str] = str(uuid.uuid4())
    title: str
    timestamp: Optional[datetime] = None
    subject_id: Optional[str] = None
    lessons: Optional[List[Lesson]] = []


class TopicDto(Topic):
    age: Optional[datetime] = datetime


class SubjectDto(BaseModel):
    id: Optional[str] = str(uuid.uuid4())
    title: str  # Biology
    class_id: Optional[str] = None  # Grade
    grade: Optional[GradeLevel] = None
    term: Term
    topics: Optional[List[Topic]] = None
    lesson_count: Optional[int] = 0


class ClassroomDto(BaseModel):
    id: Optional[str] = str(uuid.uuid4())
    title: str
    subjects: Optional[List[SubjectDto]]
    # teachers: List[Teacher]  # ManyToMany relationship with Teacher


class Review(BaseModel):
    id: Optional[str] = str(uuid.uuid4())
    title: str
    review: str
    rating: float
    reviewer: str
    created_at: datetime
    subject: SubjectDto



"""Copyright 2024 Everlasting Systems and Solutions LLC (www.myeverlasting.net).
All Rights Reserved.

No part of this software or any of its contents may be reproduced, copied, modified or adapted, without the prior written consent of the author, unless otherwise indicated for stand-alone materials.

For permission requests, write to the publisher at the email address below:
office@myeverlasting.net

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""
from datetime import datetime, date
from pydantic import BaseModel, EmailStr,Field
from espy_contact.util.enums import AccessRoleEnum, StatusEnum
from typing import List, Optional, Union
import uuid
class ReachBase(BaseModel):
    id: str
    timestamp: datetime
class SeoRequest(BaseModel):
    url: str = Field(min_length=4, description="Your website url")
class WebbuilderRequest(BaseModel):
    id: str
    content: str
    product_id: int

class UserResponse(BaseModel):
    id: Optional[Union[str, int]] = str(uuid.uuid4())
    timestamp: Optional[datetime] = None
    first_name: str
    last_name: str
    email: EmailStr
    is_active: bool = False
    status: StatusEnum = StatusEnum.NEW
    role: List[AccessRoleEnum]
class AddressDto(BaseModel):
    id: Optional[str] = None
    street: str
    city: str
    state: str
    zip_code: int
    email: Optional[EmailStr] = None
    phone_number: Optional[str] = None
    country: str


class AppuserDto(UserResponse):
    password: str
    address: AddressDto

    class Config:
        from_attributes = True


class EnrollmentDto(UserResponse):
    dob: date
    gender: str
    nationality: str
    address: AddressDto
    parent_email: str
    current_school: str
    current_class: str
    achievements: str
    extracurricular: str
    parent_phone: str
    parent_name: str
    parent_occupation: str
    religion: str
    password: Optional[str] = None
    photo: Optional[Union[str, bytes]] = None
    birth_certificate: Optional[Union[str, bytes]] = None
    signature: str
    is_paid: Optional[bool] = False

    class Config:
        from_attributes = True


class SchoolDto(BaseModel):
    id: Optional[str] = str(uuid.uuid4())
    name: str
    address: AddressDto
    owner: str


class SchoolResponse(BaseModel):
    id: str
    name: str
    create_at: datetime
    address_id: str
    owner_id: str


class AcademicHistory(ReachBase):
    """Student or teacher can have multiple AcademicHistory."""

    school_name: str
    start_date: str
    end_date: str
    grade_level: str
    reason_for_leaving: str
    classroom: str  # ForeignKey to Classroom or String
    owner: AppuserDto  # ForeignKey to StudentProfile or None

    # teacher: Teacher  # Optional ForeignKey to Teacher (null allowed) or None
    class Config:
        from_attributes = True


"""Copyright 2024 Everlasting Systems and Solutions LLC (www.myeverlasting.net).
All Rights Reserved.

No part of this software or any of its contents may be reproduced, copied, modified or adapted, without the prior written consent of the author, unless otherwise indicated for stand-alone materials.

For permission requests, write to the publisher at the email address below:
office@myeverlasting.net

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""
import enum


class ResourceEnum(enum.Enum):
    POLL = "Poll"
    FORM = "Form"
    QUESTIONNAIRE = "QUESTIONNAIRE"
    TEXT = "Text"
    VIDEO = "Video"
    AUDIO = "AUDIO"
    FILE = "File"
    LINK = "Link"


class AccessRoleEnum(enum.Enum):
    ADMIN = "Admin"
    USER = "User"
    GUEST = "Guest"
    PARENT = "Parent"
    TEACHER = "Teacher"
    ACCOUNT = "Account"
    REFEREE = "Referee"
    STUDENT = "Student"


class GradeLevel(enum.Enum):
    PRESCHOOL = "PRESCHOOL"
    DAYCARE = "DAYCARE"
    KG1 = "Kindgergaten 1"
    KG2 = "Kindergaten 2"
    KG3 = "Kindgergaten 3"
    PRIMARY1 = "Primary 1"
    PRIMARY2 = "Primary 2"
    PRIMARY3 = "Primary 3"
    PRIMARY4 = "Primary 4"
    PRIMARY5 = "Primary 5"
    PRIMARY6 = "Primary 6"
    JSS1 = "Junior Secondary School 1 (JSS1)"
    JSS2 = "Junior Secondary School 2 (JSS2)"
    JSS3 = "Junior Secondary School 1 (JSS3)"
    SSS1 = "Senior Secondary School 1 (SSS1)"
    SSS2 = "Senior Secondary School 2 (SSS2)"
    SSS3 = "Senior Secondary School 3 (SSS3)"
    POST_UTME = "Post UTME (Post JAMB)"
    PREDEGREE = "Pre-degree"
    OND1 = "Ordinary National Diploma 1 (OND)"
    OND2 = "Ordinary National Diploma 2 (OND2)"
    HND1 = "Higher National Diploma 1 (HND1)"
    HND2 = "Higher National Diploma 2 (HND2)"
    YEAR1 = "One Hunder Level (University)"
    YEAR2 = "Two Hunder Level (University)"
    YEAR3 = "Three Hunder Level (University)"
    YEAR4 = "Four Hunder Level (University)"
    YEAR5 = "Five Hunder Level (University/Postgraduate)"
    YEAR6 = "Six Hunder Level (University/Postgraduate)"
    YEAR7 = "Seven Hunder Level (University/Postgraduate)"
    YEAR8 = "Eight Hunder Level (University/Postgraduate)"


class Term(enum.Enum):
    FIRST = "First"
    SECOND = "Second"
    THIRD = "Third"
    FOURTH = "Fourth"


class StatusEnum(enum.Enum):
    NEW = "New"
    ENROLLED = "Enrolled"
    ADMITTED = "Admitted"
    DEBTOR = "Debtor"
    DELETED = "Deleted"
    SUSPENDED = "Suspended"
    EXPELLED = "Expelled"


class NigerianBank(enum.Enum):
    ACCESS_BANK = "Access Bank"
    CITIBANK = "Citibank"
    DIAMOND_BANK = "Diamond Bank"
    ECOBANK_NIGERIA = "Ecobank Nigeria"
    FIDELITY_BANK = "Fidelity Bank"
    FIRST_BANK_OF_NIGERIA = "First Bank of Nigeria"
    FIRST_CITY_MONUMENT_BANK = "First City Monument Bank"
    GUARANTY_TRUST_BANK = "Guaranty Trust Bank"
    HERITAGE_BANK_PLC = "Heritage Bank Plc"
    KEYSTONE_BANK_LIMITED = "Keystone Bank Limited"
    POLARIS_BANK = "Polaris Bank"
    PROVIDUS_BANK = "Providus Bank"
    STANBIC_IBTC_BANK = "Stanbic IBTC Bank"
    STANDARD_CHARTERED = "Standard Chartered"
    STERLING_BANK = "Sterling Bank"
    SUNTRUST_BANK_NIGERIA = "Suntrust Bank Nigeria"
    UNION_BANK_OF_NIGERIA = "Union Bank of Nigeria"
    UNITED_BANK_FOR_AFRICA = "United Bank for Africa"
    UNITY_BANK_PLC = "Unity Bank Plc"
    WEMA_BANK = "Wema Bank"
    ZENITH_BANK = "Zenith Bank"




from sqlalchemy import Column, DateTime, ForeignKey, Enum, JSON,Date
from sqlalchemy.orm import relationship
from sqlalchemy.sql import func
from sqlalchemy.sql.sqltypes import String, Float,Integer, Boolean
from sqlalchemy.dialects.postgresql import ARRAY
from espy_contact.util.enums import ResourceEnum, GradeLevel, Term,AccessRoleEnum,StatusEnum
from espy_contact.util.db import Base
import uuid


class Resource(Base):
    """Type of resource can be Poll, Form Builder, Questionnaire, RichText, Video, Audio, File, Hyperlink."""

    __tablename__ = "resources"
    id = Column(String, primary_key=True, index=True)
    timestamp = Column(DateTime(), server_default=func.now())
    title = Column(String)
    type = Column(Enum(ResourceEnum))
    lesson_id = Column(String, ForeignKey("lessons.id"))
    lesson = relationship("Lesson", foreign_keys=[lesson_id])


# class QuizOption(Base):
#     __tablename__ = "quiz_options"
#     id = Column(String, primary_key=True, index=True)
#     option_text = Column(String)
#     quiz_id = Column(String, ForeignKey("quizzes.id"))
#     quiz = relationship("Quiz", back_populates="options", foreign_keys=[quiz_id])

# class Quiz(Base):
#     __tablename__ = "quizzes"
#     id = Column(String, primary_key=True, index=True)
#     question = Column(String)
#     options = relationship("QuizOption", back_populates="quiz",foreign_keys=[QuizOption.quiz_id])
#     answer_id = Column(String, ForeignKey("quiz_options.id"))
#     answer = relationship("QuizOption",foreign_keys=[answer_id])
#     lesson_id = Column(String, ForeignKey("lessons.id"))
#     lesson = relationship("Lesson", foreign_keys=[lesson_id])


class Quiz(Base):
    __tablename__ = "quizzes"

    id = Column(String, primary_key=True, default=lambda: str(uuid.uuid4()))
    title = Column(String)
    question = Column(String)
    options = Column(JSON)  # Store options as JSON
    answer = Column(String)
    lesson_id = Column(String, ForeignKey("lessons.id"))
    Lesson = relationship("Lesson", foreign_keys=[lesson_id])


class LessonNote(Base):
    __tablename__ = "lesson_notes"
    id = Column(String, primary_key=True, index=True)
    content = Column(String)
    title = Column(String)
    lesson = relationship("Lesson", back_populates="note")


class LessonResource(Base):
    __tablename__ = "lesson_resource"

    lesson_id = Column(String, ForeignKey("lessons.id"), primary_key=True)
    resource_id = Column(String, ForeignKey("resources.id"), primary_key=True)


class LessonAssets(Base):
    __tablename__ = "lessons_assets"
    asset = Column(String)
    lesson_id = Column(String, ForeignKey("lessons.id"), primary_key=True)
    lesson = relationship("Lesson", back_populates="assets")


class Lesson(Base):
    __tablename__ = "lessons"
    id = Column(String, primary_key=True, index=True, default=lambda: str(uuid.uuid4()))
    title = Column(String)
    note_id = Column(String, ForeignKey("lesson_notes.id"))
    note = relationship("LessonNote", back_populates="lesson")
    assets = relationship(
        "LessonAssets", back_populates="lesson"
    )  # Consider using a separate table for assets
    topic_id = Column(String, ForeignKey("topics.id"))
    quiz_id = Column(String, ForeignKey("quizzes.id"))
    quiz = relationship("Quiz", foreign_keys=[quiz_id])


class Topic(Base):
    __tablename__ = "topics"
    id = Column(String, primary_key=True, index=True, default=str(uuid.uuid4()))
    title = Column(String)
    lessons = relationship("Lesson", backref="topic")
    subject_id = Column(String, ForeignKey("subjects.id"))
    timestamp = Column(DateTime(), server_default=func.now())


class Subject(Base):
    __tablename__ = "subjects"
    id = Column(String, primary_key=True, index=True, default=str(uuid.uuid4()))
    timestamp = Column(DateTime(), server_default=func.now())
    title = Column(String)
    grade = Column(Enum(GradeLevel))
    topics = relationship("Topic", backref="subject")
    term = Column(Enum(Term))
    reviews = relationship("Review", backref="reviewed_subject")
    class_id = Column(String, ForeignKey("classrooms.id"))
    classroom = relationship("Classroom", back_populates="subjects")


class Classroom(Base):
    __tablename__ = "classrooms"
    id = Column(String, primary_key=True, index=True, default=str(uuid.uuid4()))
    title = Column(String, unique=True)
    timestamp = Column(DateTime(), server_default=func.now())
    teacher_id = Column(String, ForeignKey("appusers.id"), nullable=True)
    teacher = relationship("Appuser")  # Optional backref
    school_id = Column(String)

    subjects = relationship("Subject", back_populates="classroom")
    students = relationship("Student", back_populates="classroom")


class Review(Base):
    __tablename__ = "reviews"
    id = Column(String, primary_key=True, index=True, default=str(uuid.uuid4()))
    timestamp = Column(DateTime(), server_default=func.now())
    title = Column(String)
    review = Column(String)
    rating = Column(Float)
    reviewer = Column(String)
    created_at = Column(DateTime)
    subject_id = Column(String, ForeignKey("subjects.id"))
    # reviewed_subject = relationship("Subject", foreign_keys=[subject_id])


class ClassroomSubject(Base):
    __tablename__ = "classroom_subjects"
    id = Column(String, primary_key=True, index=True, default=str(uuid.uuid4()))
    classroom_id = Column(String, ForeignKey("classrooms.id"))
    subject_id = Column(String, ForeignKey("subjects.id"))

# class ReachBase:
#     id = Column(String, primary_key=True, index=True)
#     timestamp = Column(DateTime(), server_default=func.now())
# address_associations = Table(
#     "address_associations", Base.metadata,
#     Column("id", String, primary_key=True, index=True),
#     Column("school_id", String, ForeignKey("schools.id")),
#     Column("user_id", String, ForeignKey("appusers.id")),
#     Column("address_id", String, ForeignKey("addresses.id")),
# )
class Address(Base):
    __tablename__ = "addresses"
    id = Column(String, primary_key=True, index=True, default=lambda: str(uuid.uuid4()))
    street = Column(String)
    city = Column(String)
    state = Column(String)
    zip_code = Column(Integer)
    email = Column(String)
    phone_number = Column(String)
    country = Column(String)


class Appuser(Base):
    __tablename__ = "appusers"
    id = Column(String, primary_key=True, index=True, default=lambda: str(uuid.uuid4()))
    timestamp = Column(DateTime(), server_default=func.now())
    first_name = Column(String)
    last_name = Column(String)
    country = Column(String)
    email = Column(String, nullable=False, unique=True)
    password = Column(String)
    token = Column(String)
    dp = Column(String)
    is_active = Column(Boolean)
    roles = Column(ARRAY(Enum(AccessRoleEnum)))
    status = Column(Enum(StatusEnum))
    address_id = Column(
        String, ForeignKey("addresses.id"), nullable=True
    )  # Add foreign key
    address = relationship(
        "Address", uselist=False, backref="user"
    )  # One-to-one relationship


class Enrollment(Base):
    __tablename__ = "enrollments"
    id = Column(Integer, primary_key=True, index=True, autoincrement=True)
    dob = Column(Date)
    gender = Column(String)
    nationality = Column(String)
    user_id = Column(String, ForeignKey("appusers.id"))
    user = relationship("Appuser", foreign_keys=[user_id])
    parent_email = Column(String)
    current_school = Column(String)
    current_class = Column(String)
    achievements = Column(String)
    extracurricular = Column(String)
    parent_phone = Column(String)
    parent_name = Column(String)
    parent_occupation = Column(String)
    religion = Column(String)
    grade_level = Column(Enum(GradeLevel))
    term = Column(Enum(Term))
    academic_year = Column(Integer)
    remarks = Column(String)
    photo = Column(String)
    birth_certificate = Column(String)
    signature = Column(String)
    is_paid = Column(Boolean)


class School(Base):
    __tablename__ = "schools"
    id = Column(String, primary_key=True, index=True, default=lambda: str(uuid.uuid4()))
    name = Column(String)
    owner_id = Column(String, ForeignKey("appusers.id"))
    timestamp = Column(DateTime(), server_default=func.now())
    address_id = Column(
        String, ForeignKey("addresses.id"), nullable=True
    )  # Add foreign key
    address = relationship(
        "Address", uselist=False, backref="school"
    )  # One-to-one relationship


class Student(Base):
    __tablename__ = "student"
    id = Column(String, primary_key=True, index=True, default=lambda: str(uuid.uuid4()))
    timestamp = Column(DateTime(), server_default=func.now())
    biodata_id = Column(
        String, ForeignKey("appusers.id")
    )  # Define ForeignKey for relationship
    biodata = relationship("Appuser")  # Define relationship
    date_of_birth = Column(String)
    id_card = Column(String)
    class_id = Column(String, ForeignKey("classrooms.id"))
    classroom = relationship("Classroom", foreign_keys=[class_id])


class Student_attendance(Base):
    __tablename__ = "attendance"
    id = Column(String, primary_key=True, index=True, default=lambda: str(uuid.uuid4()))
    timestamp = Column(DateTime(), server_default=func.now())
    student_id = Column(String, ForeignKey("student.id"))
    is_present = Column(Boolean, default=False)
    remarks = Column(String)
    created_by = Column(String)  # email of the teacher who makrked the attendance


class EnrollmentRequest(Base):
    __tablename__ = "enrollment"
    id = Column(String, primary_key=True, index=True, default=lambda: str(uuid.uuid4()))
    timestamp = Column(DateTime(), server_default=func.now())
    student_id = Column(
        String, ForeignKey("student.id")
    )  # Define ForeignKey for relationship
    student_profile = relationship("Student")
    grade_level = Column(String)
    academic_year = Column(Integer)
    remarks = Column(String)


class AcademicHistory(Base):
    """Student or teacher can have multiple AcademicHistory."""

    __tablename__ = "academic_history"
    id = Column(String, primary_key=True, index=True, default=lambda: str(uuid.uuid4()))
    timestamp = Column(DateTime(), server_default=func.now())
    school_name = Column(String)
    start_date = Column(DateTime)
    end_date = Column(String)
    grade_level = Column(String)
    reason_for_leaving = Column(String)
    classroom = Column(String)
    owner_id = Column(String, ForeignKey("appusers.id"))
    owner = relationship("Appuser", backref="academic_history")


# Additional model for assignment relationship (optional):
# class Class_Teacher(Base):
#     __tablename__ = "class_teacher"
#     id = Column(String, primary_key=True, index=True)
#     timestamp = Column(DateTime(), server_default=func.now())
#     classroom_id= Column(Integer, ForeignKey("classrooms.id"))  # ForeignKey to Classroom
#     teacher= Column(Integer, ForeignKey("appuser.id"))  # ForeignKey to Teacher



from espy_contact.util.db import Base
from sqlalchemy import Column, DateTime
from sqlalchemy.sql import func
from sqlalchemy.sql.sqltypes import Integer, String
class Message(Base):
    """Model for in-app messaging."""
    __tablename__ = "messages"
    id = Column(Integer, primary_key=True, index=True)
    sender = Column(String)
    recipient = Column(String)
    content = Column(String)
    timestamp = Column(DateTime(), server_default=func.now())
    def as_dict(self):
        return {c.name: getattr(self, c.name) for c in self.__table__.columns}


from espy_contact.util.db import Base
from espy_contact.util.enums import StatusEnum
from datetime import datetime
from sqlalchemy import Column, DateTime, ForeignKey, Text, Date, Float,Enum
from sqlalchemy.sql.sqltypes import Integer, String, Boolean
from sqlalchemy.orm import relationship


class Tranx(Base):
    __tablename__ = "tranx"

    ref = Column(String, primary_key=True)
    stripeId = Column(String, nullable=False)
    amount = Column(Integer, nullable=False)
    currency = Column(String, nullable=False)
    status = Column(String, nullable=False)
    payee_id = Column(String, ForeignKey("appusers.id"))
    detail = Column(Text)  # Text allows for larger details if needed
    createdOn = Column(DateTime, default=datetime)
    modifiedOn = Column(DateTime, onupdate=datetime)

    payee = relationship("Appuser", backref="transactions")


class Account(Base):
    __tablename__ = "accounts"

    id = Column(Integer, primary_key=True)
    bank = Column(String, nullable=False)  # Assuming NigerianBank is a string enum
    account_name = Column(String, nullable=False)
    account_number = Column(String, nullable=False)
    currency = Column(String, nullable=False)
    is_active = Column(Boolean, default=True)
    account_officer = Column(String)
    account_admin = Column(String)
    created_on = Column(DateTime, default=datetime)
    modified_on = Column(DateTime, onupdate=datetime)


class Fee(Base):
    __tablename__ = "fees"

    id = Column(Integer, primary_key=True)
    classroom_id = Column(String, ForeignKey("classrooms.id"))
    name = Column(String, nullable=False)
    amount = Column(Float, nullable=False)
    due_date = Column(Date, nullable=False)
    start_date = Column(Date, nullable=False)
    status = Column(Enum(StatusEnum))

    classroom = relationship("Classroom")



from pydantic import BaseModel,Field
from typing import List,Optional,Any
from typing_extensions import Annotated
from pydantic.functional_validators import BeforeValidator
from datetime import datetime
from .schema import UserResponse
from bson import ObjectId


PyObjectId = Annotated[str, BeforeValidator(str)]
# class UserModel(BaseModel):
#     """ Horace User record."""
#     id: Optional[PyObjectId] = Field(alias="_id", default=None)
#     firstname: Optional[str] = None
#     lastname: Optional[str] = None
#     email: Optional[str] = None
#     password: Optional[str] = None
#     active: bool = False
class UserCol(BaseModel):
    users: List[UserResponse]

class Course(BaseModel):
    id: Optional[ObjectId] = Field(default_factory=ObjectId, alias='_id')
    author: UserResponse
    course_name: str = Field(..., alias='courseName')
    category: str
    target: str
    curriculum: Any
    brief: str
    price: float
    tax: float
    created_on: datetime = Field(default_factory=datetime.now, alias='createdOn')
    thumbnail: Optional[str] = None
    updated_on: Optional[datetime] = Field(None, alias='updatedOn')
    total_steps: int = Field(..., alias='totalSteps')
    draft: bool

    class Config:
        populate_by_name = True
        arbitrary_types_allowed=True
        json_schema_extra = {
            "example": {
                "author": {
                    # Assuming some example fields for Appuser
                },
                "course_name": "Introduction to Python",
                "category": "Programming",
                "target": "Beginner programmers",
                "curriculum": {
                    "introduction": "Welcome to the course",
                    "contents": [
                        {"module": "Basics", "lessons": ["Setup", "Syntax"]}
                    ]
                },
                "brief": "Learn Python from scratch!",
                "price": 19.99,
                "tax": 1.99,
                "thumbnail": "url_to_thumbnail_image",
                "total_steps": 10,
                "draft": False
            }
        }

class AssetCount(BaseModel):
    quiz_count: int
    lab_count: int
    lesson_count: int
    download_count: int
    note_count: int

class Lecture(BaseModel):
    id: int
    title: str
    video: str
    type: str
    slide: str

class Quiz(BaseModel):
    # Define the properties of Quiz here
    questions: List[str]

class Note(BaseModel):
    # Define the properties of Note here
    content: str

class Section(BaseModel):
    title: str
    description: str
    id: str
    lecture: List[Lecture]
    quiz: Optional[Quiz] = None
    note: Optional[Note] = None

class CurriculumMap(BaseModel):
    section: List[Section]
    requirement: List[str]
    objective: List[str]

class Post(BaseModel):
    id: str
    user: str
    message: str
    type: str
    course: str
    created_on: datetime = Field(default_factory=datetime.now, alias='createdOn')
    modified_on: datetime = Field(default_factory=datetime.now, alias='modifiedOn')
    like: int
    share: int
    rating: int

    class Config:
        populate_by_name = True
        json_encoders = {
            datetime: lambda dt: dt.isoformat()
        }
        json_schema_extra = {
            "example": {
                "id": "12345",
                "user": "user123",
                "message": "This is a post message.",
                "type": "info",
                "course": "course123",
                "createdOn": "2023-01-01T12:00:00Z",
                "modifiedOn": "2023-01-02T12:00:00Z",
                "like": 10,
                "share": 5,
                "rating": 4
            }
        }
class PostResponse(BaseModel):
    id: str
    user: str
    message: str
    type: str
    course: str
    created_on: datetime = Field(default_factory=datetime.now, alias='createdOn')
    modified_on: datetime = Field(default_factory=datetime.now, alias='modifiedOn')
    like: int
    share: int
    rating: int

    class Config:
        # Use alias generation to maintain the Python convention in code and the original Java naming in JSON
        populate_by_name = True
        from_attributes = True  # Enable ORM mode if integrating with ORMs
        json_schema_extra = {
            "example": {
                "id": "post123",
                "user": "user456",
                "message": "This is a sample post message.",
                "type": "announcement",
                "course": "course789",
                "createdOn": "2023-01-01T12:00:00Z",
                "modifiedOn": "2023-01-02T12:00:00Z",
                "like": 100,
                "share": 50,
                "rating": 5
            }
        }

class CourseResponse(BaseModel):
    course_id: str
    course_name: str
    author: UserResponse
    category: str
    target: str
    curriculum: CurriculumMap
    brief: str
    price: float
    tax: float
    created_on: str
    thumbnail: str
    total_steps: int
    draft: bool
    posts: List[PostResponse]
    asset_count: AssetCount

class CourseItem(BaseModel):
    id: Optional[PyObjectId] = Field(alias="id", default=None)
    course_name: Optional[str] = None
    brief: str
    updated_on: Optional[datetime] = None
    thumbnail: Optional[str] = None
    category: Optional[str]
    total_steps: Optional[int] = 0
    draft: Optional[bool]
    author: Optional[UserResponse] = None
    students: Optional[int] = 0
    posts: Optional[List[Post]] = None
    cost: Optional[float] = 0

class Post(BaseModel):
    # Define Post model attributes as needed
    pass

class Author(BaseModel):
    first_name: str
    last_name: str

    def full_name(self):
        return f"{self.first_name} {self.last_name}"





from setuptools import setup, find_packages

VERSION = '0.0.2'
DESCRIPTION = 'ESSL users management'
LONG_DESCRIPTION = 'Internal helper package for ESSL users management'
setup(
    name='espy_contact',
    version='0.4',
    packages=find_packages(),
    install_requires=[
        'bcrypt==4.1.2',
        'pytest==8.1.1',
        'pydantic==2.7.1',
        'sqlalchemy==2.0.29',
        'bson==0.5.10'
        ],
    author='Femi Adigun',
    author_email='femi.adigun@myeverlasting.net',
    description=DESCRIPTION,
    long_description_content_type='text/markdown',
    long_description=LONG_DESCRIPTION,
    keywords=['fastapi','ESSL','ReachAI']
)


from datetime import datetime, date
from typing import Optional, Union
from pydantic import BaseModel
from espy_contact.util.enums import StatusEnum, NigerianBank


class TranxDto(BaseModel):
    id: Optional[int] = None
    ref: str
    stripeId: str
    amount: float
    currency: str
    status: StatusEnum
    payee_id: Optional[str]  # Assuming Appuser has an ID field
    detail: Optional[Union[dict, str]]  # Can be dict or string for flexibility
    createdOn: Optional[datetime]
    modifiedOn: Optional[datetime]


class AccountDto(BaseModel):
    id: Optional[int] = None
    bank: NigerianBank
    account_name: str
    account_number: str
    currency: str
    is_active: bool
    account_officer: str
    account_admin: str
    created: datetime
    modified: datetime


class FeeDto(BaseModel):
    classroom_id: str
    fee_name: str
    amount: float
    due_date: date
    start_date: date
    status: StatusEnum
